#
# Copyright 2013 (c) John Colanduoni
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

require 'RMagick'

module Novector
  class VectorFallbackController < ActionController::Base
    public
    
    def render_fallback
      width = Integer(params[:width])
      height = Integer(params[:height])
      source = params[:src]
      format = params[:format]
      
      raise 'Asset is not an SVG file.' unless source =~ /\.svg$/
      raise 'Unsupported output format.' unless ['png', 'gif'].include? format
      
      # Open asset
      asset = Rails.application.assets.find_asset source
      raise ActionController::RoutingError.new('Asset not found') unless asset
      asset_mtime = File.mtime(asset.pathname)
    
      # If the asset is up to date on the client, don't waste the bandwidth
      last_time = Time.rfc2822(request.env['HTTP_IF_MODIFIED_SINCE']) rescue nil
      if last_time and asset_mtime  < last_time
        render :text => '304 Not Modified', :status => 304
        return
      end
      
      cache_name = '#{source}_#{asset_mtime}_#{width}_#{height}_#{format}'
      blob = Rails.cache.read(cache_name)
      unless blob
        blob = convert_svg(asset.pathname, width, height, format)
        Rails.cache.write(cache_name, blob)
      end
    
      # Send asset
      response.header['Last-Modified'] = asset_mtime.to_s
      send_data(blob, :type => "image/#{format}", :disposition => 'inline')
    end
    
    protected
    
    def convert_svg(source_filename, width, height, format)
      blob = nil
      IO::popen(['rsvg-convert', '--width', width.to_s, '--height', height.to_s, source_filename.to_s]) do |pipe|
        blob = pipe.read
      end
      # Convert from png if neccessary
      if format == 'png'
        return blob
      else
        png_image = (Magick::Image.from_blob(blob) { self.format = 'PNG'})[0]
        return png_image.to_blob { self.format = format.upcase; self.quality = 10 }
      end
    end
  end
end